@@ -23,6 +23,7 @@ import random
 import struct
 
 import six
+import time
 
 import pyghmi.exceptions as pygexc
 
@@ -50,7 +51,16 @@ class Unsupported(Exception):
 def fromstring(inputdata):
     if b'!entity' in inputdata.lower():
         raise Exception('Unsupported XML')
-    return etree.fromstring(inputdata)
+    try:
+        return etree.fromstring(inputdata)
+    except etree.XMLSyntaxError:
+        inputdata = bytearray(inputdata.decode('utf8', errors='backslashreplace').encode())
+        for i in range(len(inputdata)):
+            if inputdata[i] < 0x20 and inputdata[i] not in (9, 0xa, 0xd):
+                inputdata[i] = 63
+        inputdata = bytes(inputdata)
+        return etree.fromstring(inputdata)
+
 
 
 def run_command_with_retry(connection, data):
@@ -271,20 +281,35 @@ class LenovoFirmwareConfig(object):
             cfgfilename = "config"
         options = {}
         data = None
-        rsp = self.xc.grab_redfish_response_with_status(
-                '/redfish/v1/Managers/1')
+        if not hasattr(self.xc, 'grab_redfish_response_with_status'):
+            rsp = ({}, 500)
+        elif self.connection:
+            rsp = ({}, 200)
+        else:
+            rsp = self.xc.grab_redfish_response_with_status(
+                    '/redfish/v1/Managers/1')
         if rsp[1] == 200:
             if 'purley' not in rsp[0].get('Oem', {}).get('Lenovo', {}).get(
-                                    'release_name', 'purley'):
+                                    'release_name', 'unknown'):
                 rsp = self.xc.grab_redfish_response_with_status(
                     '/redfish/v1/Systems/1/Actions/Oem/LenovoComputerSystem.DSReadFile',
                     {'Action': 'DSReadFile', 'FileName': cfgfilename})
             else:
                 rsp = (None, 500)
         if rsp[1] == 200:
-            data = rsp[0]['Content']
-            data = base64.b64decode(data)
-            data = EfiCompressor.FrameworkDecompress(data, len(data))
+            for _ in range(0, 30):
+                data = rsp[0]['Content']
+                data = base64.b64decode(data)
+                data = EfiCompressor.FrameworkDecompress(data, len(data))
+                if len(data) != 0:
+                    break
+                if self.connection:
+                    self.connection.ipmi_session.pause(2)
+                else:
+                    time.sleep(2)
+                rsp = self.xc.grab_redfish_response_with_status(
+                    '/redfish/v1/Systems/1/Actions/Oem/LenovoComputerSystem.DSReadFile',
+                    {'Action': 'DSReadFile', 'FileName': cfgfilename})
         else:
             if self.connection is None:
                 raise Unsupported('Not Supported')
@@ -335,10 +360,12 @@ class LenovoFirmwareConfig(object):
                             readonly = 'true'
                     possible = []
                     current = None
+                    currentidxes = []
                     default = None
                     reset = False
                     name = setting.find("mriName").text
                     help = setting.find("desc").text
+                    validexpression = None
                     onedata = setting.find('text_data')
                     if onedata is not None:
                         if onedata.get('password') == 'true':
@@ -352,11 +379,19 @@ class LenovoFirmwareConfig(object):
                     if onedata is not None:
                         if onedata.get('maxinstance') is not None:
                             forceinstance = True
+                        validexpression = onedata.get('pattern', None)
                         instances = list(onedata.iter('instance'))
                         if not instances:
                             protect = True  # not supported yet
                         else:
-                            current = [x.text for x in instances]
+                            instbynum = {}
+                            defidx = 1
+                            for x in instances:
+                                xid = int(x.get('ID', defidx))
+                                instbynum[xid] = x
+                                defidx += 1
+                            current = [instbynum[idx].text for idx in sorted(instbynum)]
+                            currentidxes = list(sorted(instbynum))
                         default = onedata.get('default', None)
                         if default == '':
                             default = None
@@ -418,6 +453,8 @@ class LenovoFirmwareConfig(object):
                         if current and len(current) > 1:
                             instidx = 1
                             for inst in current:
+                                if currentidxes:
+                                    instidx = currentidxes.pop(0)
                                 optname = '{0}.{1}'.format(optionname, instidx)
                                 options[optname] = dict(
                                     current=inst,
@@ -437,6 +474,7 @@ class LenovoFirmwareConfig(object):
                                     readonly_expression=readonly,
                                     hide_expression=hide,
                                     sortid=sortid,
+                                    validexpression=validexpression,
                                     alias=alias)
                                 sortid += 1
                                 instidx += 1
@@ -465,6 +503,7 @@ class LenovoFirmwareConfig(object):
                                 readonly_expression=readonly,
                                 hide_expression=hide,
                                 sortid=sortid,
+                                validexpression=validexpression,
                                 alias=alias)
                             sortid += 1
                         continue
@@ -489,6 +528,7 @@ class LenovoFirmwareConfig(object):
                                                readonly_expression=readonly,
                                                hide_expression=hide,
                                                sortid=sortid,
+                                               validexpression=validexpression,
                                                alias=alias)
                     sortid = sortid + 1
         for opt in options:
